/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package avgproducerconsumer;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.util.Scanner;

/**
 *
 * @author bellini
 */
public class AvgProducerConsumer {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        try {
            SharedBuffer sb1=new SharedBuffer(10);
            SharedBuffer sb2=new SharedBuffer(10);
            FileInputThread ift=new FileInputThread("input.txt", sb1);
            AvgThread avg=new AvgThread(sb1, sb2);
            FileOutputThread oft=new FileOutputThread("output.txt", sb2);
            ift.start();
            oft.start();
            avg.start();
        } catch(FileNotFoundException e) {
            e.printStackTrace();
        }
    }
    
}


class SharedBuffer {
    private Object[] data;
    private int insPos;
    private int extPos;
    private int count;
    
    public SharedBuffer(int n) {
        data=new Object[n];
        count=0;
        insPos=extPos=0;
    }
    
    public synchronized void add(Object o) throws InterruptedException {
        while(count==data.length)
            wait();
        data[insPos]=o;
        insPos = (insPos+1)%data.length;
        count++;
        notifyAll();
    }
    
    public synchronized Object extract() throws InterruptedException {
        while(count==0)
            wait();
        Object r=data[extPos];
        extPos=(extPos+1)%data.length;
        count--;
        notifyAll();
        return r;
    }
}


class FileInputThread extends Thread {
    private InputStream is;
    private SharedBuffer sb;
    
    public FileInputThread(String file, SharedBuffer sb) throws FileNotFoundException {
        this.sb=sb;
        is = new FileInputStream(file);
    }
    
    public void run() {
        try {
            Scanner s=new Scanner(is);
            while(s.hasNext()) {
               double v=s.nextDouble();
               sb.add(v);
            }
            sb.add(null);
        } catch(InterruptedException e) {
            
        }
    }
}

class FileOutputThread extends Thread {
    private OutputStreamWriter osw;
    private SharedBuffer sb;
    
    public FileOutputThread(String file, SharedBuffer sb) throws FileNotFoundException {
        this.sb=sb;
        osw=new OutputStreamWriter(new FileOutputStream(file));
    }
    
    public void run() {
        try {
            Object v;
            do {
                v=sb.extract();
                System.out.println("v="+v);
                if(v!=null) {
                    osw.write(v.toString()+"\r\n");
                    osw.flush();
                }
            } while(v!=null);
            osw.close();
        } catch(InterruptedException e) {
            
        } catch(IOException e){
            
        }
    }
}

class AvgThread extends Thread {
    private SharedBuffer in;
    private SharedBuffer out;
    public AvgThread(SharedBuffer i, SharedBuffer o) {
        this.in=i;
        this.out=o;
    }
    
    public void run() {
        try {
            Double v1,v2=null;
            do {
                v1=(Double)in.extract();
                if(v1!=null) {
                    v2=(Double)in.extract();
                    if(v2!=null)
                        out.add((v1+v2)/2);
                    else
                        out.add(null);
                }
                else
                    out.add(null);
            } while(v1!=null && v2!=null);
        } catch(InterruptedException e){
            
        }
    }
}