/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cyclicthreads;

import java.util.concurrent.Semaphore;

/**
 *
 * @author bellini
 */
public class CyclicThreads {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        // TODO code application logic here
        final int n1=10,n2=5;
        Synchronizer s=new SynchronizerImpl2();
        Coordinator c1=new CoordinatorImpl2(n1);
        for(int i=0;i<n1; i++)
            new Worker(i, 0, c1, s).start();
        
        Coordinator c2=new CoordinatorImpl2(n2);
        for(int i=0;i<n2; i++)
            new Worker(i, 1, c2, s).start();
    }
    
}

interface Coordinator {
    void waitTurn(int id) throws InterruptedException;
    void next(int id);
}

class CoordinatorImpl1 implements Coordinator {
    private Semaphore[] s;
    
    public CoordinatorImpl1(int n) {
        s=new Semaphore[n];
        for(int i=0; i<n; i++)
            s[i]=new Semaphore(i==0 ? 1 : 0);
    }
    
    public void waitTurn(int idThread) throws InterruptedException {
        s[idThread].acquire();
    }
    
    public void next(int idThread) {
        s[(idThread+1)%s.length].release();
    }
}

class CoordinatorImpl2 implements Coordinator {
    private int turn;
    private int n;
    
    public CoordinatorImpl2(int n) {
        this.turn=0;
        this.n=n;
    }
    
    public synchronized void waitTurn(int id) throws InterruptedException {
        while(turn!=id)
            wait();
    }
    
    public synchronized void next(int id) {
        turn=(turn+1)%n;
        notifyAll();
    }
}

interface Synchronizer {
    void synch(int cycle) throws InterruptedException;
}

class SynchronizerImpl1 implements Synchronizer {
    private Semaphore[] s;
    public SynchronizerImpl1() {
        s=new Semaphore[2];
        s[0]=new Semaphore(0);
        s[1]=new Semaphore(0);
    }
    
    public void synch(int cycle) throws InterruptedException {
        s[cycle].release();
        s[1-cycle].acquire();
    }
}

class SynchronizerImpl2 implements Synchronizer {
    private int count;

    public SynchronizerImpl2() {
        count=2;
    }
    
    public synchronized void synch(int cycle) throws InterruptedException {
        count--;
        while(count>0 && count<2)
            wait();
        notifyAll();
        count=2;
    }
}

class Worker extends Thread {
    private Coordinator c;
    private int id;
    private int cycle;
    private Synchronizer s;
    Worker(int id, int cycle, Coordinator c, Synchronizer s) {
        this.c=c;
        this.id=id;
        this.cycle=cycle;
        this.s=s;
    }
    
    @Override
    public void run() {
        int nc=0;
        try {
            while(true) {
                c.waitTurn(id);
                if(id==0) {
                    s.synch(cycle);
                }
                System.out.println((nc++)+": thread "+id+"/"+cycle+" è il mio turno!");
                Thread.sleep((int)(Math.random()*1000));
                c.next(id);
            }
        } catch(InterruptedException e) {
            
        }
    }
}